@@ -12,8 +12,11 @@
     !defined(EVENT_PORT) &&     \
     !defined(EVENT_NOEVENT)
 #ifdef OS_WIN
-// #define EVENT_IOCP // IOCP improving
-#define EVENT_POLL
+  #if WITH_WEPOLL
+    #define EVENT_EPOLL // wepoll -> iocp
+  #else
+    #define EVENT_POLL  // WSAPoll
+  #endif
 #elif defined(OS_LINUX)
 #define EVENT_EPOLL
 #elif defined(OS_MAC)
